@@ -291,7 +291,7 @@ def escape(obj):
 	chars = u""
 	if isinstance(obj, Exception):
 		for char in obj.object[obj.start:obj.end]:
-			chars += subst.get(char, u"\\u" % hex(ord(char))[2:])
+			chars += subst.get(char, u"\\u%s" % hex(ord(char))[2:].rjust(4, '0'))
 		return chars, obj.end
 	else:
 		return obj.encode("ASCII", "escape")
@@ -303,7 +303,7 @@ def make_ascii_printable(text, subst=""):
 	return "".join([char if char in ascii_printable else subst for char in text])
 
 
-def make_filename_safe(unistr, encoding=fs_enc, subst="_"):
+def make_filename_safe(unistr, encoding=fs_enc, subst="_", concat=True):
 	"""
 	Make sure unicode string is safe to use as filename.
 	
@@ -333,7 +333,10 @@ def make_filename_safe(unistr, encoding=fs_enc, subst="_"):
 			c = safe_asciize(unistr[i:i + 1])
 		uniout += c
 	# Remove invalid chars
-	uniout = re.sub(r"[\\/:*?\"<>|]+", subst, uniout)
+	pattern = r"[\\/:*?\"<>|]"
+	if concat:
+		pattern += "+"
+	uniout = re.sub(pattern, subst, uniout)
 	return uniout
 
 
@@ -369,8 +372,7 @@ def center(text, width = None):
 def create_replace_function(template, values):
 	""" Create a replace function for use with e.g. re.sub """
 	def replace_function(match, template=template, values=values):
-		for i, group in enumerate(match.groups()):
-			template = template.replace("\\%i" % (i + 1), group)
+		template = match.expand(template)
 		return template % values
 	return replace_function
 
